# -*- coding: ISO-8859-1 -*-

# Copyright (C) 2002, 2003 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import curses

import config
import window
import events, hub
import statusbar

from helper import formattime, debug

class playerwin(window.window):
    def __init__(self, screen, h, w, y, x, channel, playerid):
        self.song = None
        self.time = 0
        self.paused = 0
        self.stopped = 1
        self.playerid = playerid
        self.keybindings = config.keybindings.general
        window.window.__init__(self, screen, h, w, y, x,
                               config.colors.playerwindow,
                               _("Playback Info"))

        # we don't want to have the focus
        self.panel.bottom()

        channel.subscribe(events.playbackinfochanged, self.playbackinfochanged)
        channel.subscribe(events.keypressed, self.keypressed)
        self.update()

    def playbackinfochanged(self, event):
        if event.playbackinfo.playerid==self.playerid:
            self.song = event.playbackinfo.song
            self.paused = event.playbackinfo.ispaused()
            self.stopped = event.playbackinfo.isstopped()
            if self.song:
                self.settitle("%s%s - %s" % (event.playbackinfo.iscrossfading() and "-> " or "",
                                             self.song.artist,
                                             self.song.title))
            else:
                self.settitle(_("Playback Info"))
            self.time = event.playbackinfo.time
            self.update()

            # update player info file, if configured
            if config.general.playerinfofile:
                f = open(config.general.playerinfofile, "w")
                if self.song:
                    f.write("%s - %s (%s/%s)\n"  % ( self.song.artist,
                                                     self.song.title,
                                                     formattime(self.time),
                                                     formattime(self.song.length)))
                f.close()

    def keypressed(self, event):
        key = event.key
        if key in self.keybindings["playerstart"] and self.paused:
            hub.hub.notify(events.playerstart(self.playerid))
        elif key in self.keybindings["playerpause"] and not self.paused and not self.stopped:
            hub.hub.notify(events.playerpause(self.playerid))
        elif key in self.keybindings["playerstart"]:
            hub.hub.notify(events.playerstart(self.playerid))
        elif key in self.keybindings["playernextsong"]:
            hub.hub.notify(events.playerforward(self.playerid))   
        elif key in self.keybindings["playerstop"]:
            hub.hub.notify(events.playerstop(self.playerid))
        elif ord("1")<=key-1024<=ord("5") and self.song:
            self.song.rate(key-1024-ord("1")+1)
        else:
            return
        raise hub.TerminateEventProcessing

    def updatestatusbar(self):
        if self.song and not self.paused and self.keybindings["playerpause"]:
            sbar = statusbar.generatedescription("general", "playerpause")
        else:
            sbar = statusbar.generatedescription("general", "playerstart")
        hub.hub.notify(events.updatestatusbar(1, sbar))
        
    def update(self):
        window.window.update(self)
        self.updatestatusbar()
        self.win.addstr(1, 1, " "*self.iw)
        if self.song:
            self.win.move(1, 1)
            s1 = _("Time:")
            s2 = " %s/%s " % (formattime(self.time), formattime(self.song.length))
            self.win.addstr(s1, self.colors.description)
            self.win.addstr(s2, self.colors.content)

            if not self.paused:
                barlen = self.iw-len(s1)-len(s2)
                try:
                    percentplayed = int(barlen*self.time/self.song.length)
                except ZeroDivisionError:
                    percentplayed = 0
                self.win.addstr("#"*(percentplayed), self.colors.progressbarhigh)
                self.win.addstr("#"*(barlen-percentplayed), self.colors.progressbar)
            else:
                self.win.addstr(_("paused"), self.colors.description)
